/* Jalali - Jalali to Gregorian and reverse date converter
 * Copyright (C) 2002 Behdad Esfahbod.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public  
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License  
 * along with this library, in a file named COPYING; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA
 * 
 * For licensing issues, contact <fwpg@sharif.edu>.
 */

Int8 g_days_in_month[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
Int8 j_days_in_month[12] = { 31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29 };
Int8 g_leap_month[12] = { 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
Int8 j_leap_month[12] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };

Int16
gregorian_is_leap (Int16 y)
{
  return (!(y % 4) && ((y % 100) || !(y % 400)));
}

Int16
jalali_is_leap (Int16 y)
{
  Int16 t;
  return (t = (y + 11) % 33, !(t % 4) && t != 32);
}

void
gregorian_to_jalali (Int16 * j_y,
		     Int16 * j_m,
		     Int16 * j_d,
		     const Int16 g_y, const Int16 g_m, const Int16 g_d)
{
  Int32 gy, gm, gd;
  Int32 jy, jm, jd;
  Int32 g_day_no, j_day_no;
  Int16 j_np;

  Int16 i;
  gy = g_y - 1600;
  gm = g_m - 1;
  gd = g_d - 1;

  g_day_no = 365 * gy + (gy + 3) / 4 - (gy + 99) / 100 + (gy + 399) / 400;
  for (i = 0; i < gm; ++i)
    g_day_no += g_days_in_month[i];
  if (gm > 1 && ((gy % 4 == 0 && gy % 100 != 0) || (gy % 400 == 0)))
    /* leap and after Feb */
    ++g_day_no;
  g_day_no += gd;

  j_day_no = g_day_no - 79;

  j_np = j_day_no / 12053;
  j_day_no %= 12053;

  jy = 979 + 33 * j_np + 4 * (j_day_no / 1461);
  j_day_no %= 1461;

  if (j_day_no >= 366)
    {
      jy += (j_day_no - 1) / 365;
      j_day_no = (j_day_no - 1) % 365;
    }

  for (i = 0; i < 11 && j_day_no >= j_days_in_month[i]; ++i)
    {
      j_day_no -= j_days_in_month[i];
    }
  jm = i + 1;
  jd = j_day_no + 1;
  *j_y = jy;
  *j_m = jm;
  *j_d = jd;

  return;
}

#define div(a, b) ((a)/(b))

void
jalali_to_gregorian (Int16 * g_y,
		     Int16 * g_m,
		     Int16 * g_d,
		     const Int16 j_y, const Int16 j_m, const Int16 j_d)
{
  Int32 gy, gm, gd;
  Int32 jy, jm, jd;
  Int32 g_day_no, j_day_no;
  Int16 leap;

  Int16 i;

  jy = j_y - 979;
  jm = j_m - 1;
  jd = j_d - 1;

  j_day_no = 365 * jy + (jy / 33) * 8 + (jy % 33 + 3) / 4;
  for (i = 0; i < jm; ++i)
    j_day_no += j_days_in_month[i];

  j_day_no += jd;

  g_day_no = j_day_no + 79;
  gy = 1600 + 400 * div (g_day_no, 146097);	/* 146097 = 365*400 + 400/4 - 400/100 + 400/400 */
  g_day_no = g_day_no % 146097;
  leap = 1;
  if (g_day_no >= 36525)	/* 36525 = 365*100 + 100/4 */
    {
      g_day_no--;
      gy += 100 * div (g_day_no, 36524);	/* 36524 = 365*100 + 100/4 - 100/100 */
      g_day_no = g_day_no % 36524;

      if (g_day_no >= 365)
	g_day_no++;
      else
	leap = 0;
    }

  gy += 4 * div (g_day_no, 1461);	/* 1461 = 365*4 + 4/4 */
  g_day_no %= 1461;

  if (g_day_no >= 366)
    {
      leap = 0;

      g_day_no--;
      gy += div (g_day_no, 365);
      g_day_no = g_day_no % 365;
    }

  for (i = 0; g_day_no >= g_days_in_month[i] + (i == 1 && leap); i++)
    g_day_no -= g_days_in_month[i] + (i == 1 && leap);
  gm = i + 1;
  gd = g_day_no + 1;

  *g_y = gy;
  *g_m = gm;
  *g_d = gd;

  return;
}
